@@ -7,12 +7,24 @@ import struct
 from pathlib import Path
 from io import BytesIO
 from importlib import import_module
-from zephyrus_sc2_parser.events import *
-from zephyrus_sc2_parser.game.player import Player
+from typing import Dict
+from zephyrus_sc2_parser.events import (
+    ObjectEvent,
+    AbilityEvent,
+    SelectionEvent,
+    ControlGroupEvent,
+    UpgradeEvent,
+    CameraEvent,
+    PlayerStatsEvent,
+)
+from zephyrus_sc2_parser.dataclasses import GameData, Map
+from zephyrus_sc2_parser.game import Player
 from zephyrus_sc2_parser.gamedata.map_info import maps
+from zephyrus_sc2_parser.exceptions import MissingMmrError, PlayerCountError
 import pytz
 import logging
 
+logger = logging.getLogger(__name__)
 
 NON_ENGLISH_RACES = {
     '저그': 'Zerg',
@@ -34,13 +46,86 @@ non_english_races = {}
 for non_eng_race, eng_race in NON_ENGLISH_RACES.items():
     non_english_races[non_eng_race.encode('utf8')] = eng_race
 
-def convert_time(windows_time):
+
+def _convert_time(windows_time):
     unix_epoch_time = math.floor(windows_time/10000000)-11644473600
     replay_datetime = datetime.datetime.fromtimestamp(unix_epoch_time).replace(tzinfo=pytz.utc)
     return replay_datetime
 
 
-def create_players(player_info, events):
+def _import_gamedata(protocol):
+    protocol_name = protocol.__name__[8:]
+    unit_data = import_module(f'zephyrus_sc2_parser.gamedata.{protocol_name}.unit_data')
+    building_data = import_module(f'zephyrus_sc2_parser.gamedata.{protocol_name}.building_data')
+    ability_data = import_module(f'zephyrus_sc2_parser.gamedata.{protocol_name}.ability_data')
+    upgrade_data = import_module(f'zephyrus_sc2_parser.gamedata.{protocol_name}.upgrade_data')
+
+    return GameData(
+        unit_data.units,
+        building_data.buildings,
+        ability_data.abilities,
+        upgrade_data.upgrades,
+    )
+
+
+def _generate_initial_summary_stats(game, metadata, detailed_info, local=False):
+    summary_stats = {
+        'mmr': {1: 0, 2: 0},
+        'avg_resource_collection_rate': {
+            'minerals': {1: 0, 2: 0},
+            'gas': {1: 0, 2: 0}
+        },
+        'max_collection_rate': {1: 0, 2: 0},
+        'avg_unspent_resources': {
+            'minerals': {1: 0, 2: 0},
+            'gas': {1: 0, 2: 0}
+        },
+        'apm': {1: 0, 2: 0},
+        'spm': {1: 0, 2: 0},
+        'resources_lost': {
+            'minerals': {1: 0, 2: 0},
+            'gas': {1: 0, 2: 0}
+        },
+        'resources_collected': {
+            'minerals': {1: 0, 2: 0},
+            'gas': {1: 0, 2: 0},
+        },
+        'workers_produced': {1: 0, 2: 0},
+        'workers_killed': {1: 0, 2: 0},
+        'workers_lost': {1: 0, 2: 0},
+        'supply_block': {1: 0, 2: 0},
+        'sq': {1: 0, 2: 0},
+        'avg_pac_per_min': {1: 0, 2: 0},
+        'avg_pac_action_latency': {1: 0, 2: 0},
+        'avg_pac_actions': {1: 0, 2: 0},
+        'avg_pac_gap': {1: 0, 2: 0},
+        'race': {1: {}, 2: {}},
+    }
+
+    mmr_data = detailed_info['m_syncLobbyState']['m_userInitialData']
+    if 'm_scaledRating' not in mmr_data[0] or 'm_scaledRating' not in mmr_data[1]:
+        logger.warning('One or more players has no MMR')
+        if not local:
+            raise MissingMmrError('One or more players has no MMR. If you want to parse replays without MMR, add "local=True" as a keyword argument')
+
+    # setting winner of the game
+    for p in metadata['Players']:
+        if p['Result'] == 'Win':
+            game.winner = p['PlayerID']
+
+    # setting basic summary stats
+    for player in metadata['Players']:
+        player_id = player['PlayerID']
+        summary_stats['apm'][player_id] = player['APM']
+
+        if 'm_scaledRating' in mmr_data[player_id - 1]:
+            summary_stats['mmr'][player_id] = mmr_data[player_id - 1]['m_scaledRating']
+        else:
+            summary_stats['mmr'][player_id] = None
+    return summary_stats
+
+
+def _create_players(player_info, events, test_flag):
     # get player name and race
     # workingSetSlotId correlates to playerIDs
     players = []
@@ -82,12 +167,18 @@ def create_players(player_info, events):
     # logic for translating user_id's into playerID's
 
     # if only one player then playerID is always 0
-    if len(players) == 1:
-        logging.info('Only one player in the game')
-        # player_obj = min(players, key=lambda x: x.player_id)
-        # player_obj.player_id = events[setup_index]['m_playerId']
-        # player_obj.user_id = events[setup_index]['m_userId']
-        return None
+    if len(players) != 2:
+        if test_flag and len(players) == 1:
+            player_obj = players[0]
+            player_obj.player_id = events[setup_index]['m_playerId']
+            player_obj.user_id = events[setup_index]['m_userId']
+
+            return {
+                1: player_obj,
+            }
+
+        logger.warning('Replay does not contain exactly 2 players')
+        raise PlayerCountError(f'There are {len(players)} in the replay. Only 2 player replays are supported')
 
     # if both user_id's larger than 2 then lowest user_id first, the largest
     elif min(players) > 2:
@@ -130,23 +221,10 @@ def create_players(player_info, events):
     }
 
 
-def import_gamedata(protocol):
-    protocol_name = protocol.__name__[8:]
-    unit_data = import_module(f'zephyrus_sc2_parser.gamedata.{protocol_name}.unit_data')
-    building_data = import_module(f'zephyrus_sc2_parser.gamedata.{protocol_name}.building_data')
-    ability_data = import_module(f'zephyrus_sc2_parser.gamedata.{protocol_name}.ability_data')
-    upgrade_data = import_module(f'zephyrus_sc2_parser.gamedata.{protocol_name}.upgrade_data')
+def _get_map_info(player_info: Dict, map_name: str, network_flag: bool) -> Map:
+    game_map = Map(map_name)
 
-    return {
-        'units': unit_data.units,
-        'buildings': building_data.buildings,
-        'abilities': ability_data.abilities,
-        'upgrades': upgrade_data.upgrades,
-    }
-
-
-def get_map_info(player_info, game_map):
-    if game_map not in maps:
+    if map_name not in maps and network_flag:
         map_bytes = player_info['m_cacheHandles'][-1]
         server = map_bytes[4:8].decode('utf8').strip('\x00 ').lower()
         file_hash = binascii.b2a_hex(map_bytes[8:]).decode('utf8')
@@ -158,11 +236,11 @@ def get_map_info(player_info, game_map):
             if map_response.status_code == 200:
                 map_file = BytesIO(map_response.content)
                 break
-            logging.error(f'Could not fetch {game_map} map file. Retrying...')
+            logger.warning(f'Could not fetch {map_name} map file. Retrying')
 
         if not map_file:
-            logging.critical(f'Failed to fetch {game_map} map file')
-            return None
+            logger.error(f'Failed to fetch {map_name} map file')
+            return game_map
 
         map_archive = mpyq.MPQArchive(map_file)
         map_data = BytesIO(map_archive.read_file('MapInfo'))
@@ -178,7 +256,7 @@ def get_map_info(player_info, game_map):
         map_width = unpack_int(map_data.read(4))[0]
         map_height = unpack_int(map_data.read(4))[0]
         maps.update({
-            game_map: {
+            map_name: {
                 'width': map_width,
                 'height': map_height,
             },
@@ -189,18 +267,18 @@ def get_map_info(player_info, game_map):
             with open(map_info_path, 'w', encoding='utf-8') as map_info:
                 map_info.write(f'maps = {maps}')
         except OSError:
-            logging.warning('Could not write map details to file')
+            logger.warning('Could not write map details to file')
 
-    game_map_info = {
-        'name': game_map,
-        'width': maps[game_map]['width'],
-        'height': maps[game_map]['height'],
-    }
+    # something may have gone wrong when trying to get map details
+    # or network_flag may be False
+    if map_name in maps:
+        game_map.width = maps[map_name]['width']
+        game_map.height = maps[map_name]['height']
 
-    return game_map_info
+    return game_map
 
 
-def create_event(game, event, protocol, summary_stats):
+def _create_event(game, event, protocol, summary_stats):
     object_events = [
         'NNet.Replay.Tracker.SUnitInitEvent',
         'NNet.Replay.Tracker.SUnitDoneEvent',
@@ -216,26 +294,34 @@ def create_event(game, event, protocol, summary_stats):
 
     if event['_event'] in object_events:
         current_event = ObjectEvent(protocol, summary_stats, game, event)
+        logger.debug(f'Created new ObjectEvent at {event["_gameloop"]}')
 
     elif event['_event'] in ability_events:
-        current_event = AbilityEvent(summary_stats, game, event)
+        current_event = AbilityEvent(game, event)
+        logger.debug(f'Created new AbilityEvent at {event["_gameloop"]}')
 
     elif event['_event'] == 'NNet.Game.SSelectionDeltaEvent':
         current_event = SelectionEvent(game, event)
+        logger.debug(f'Created new SelectionEvent at {event["_gameloop"]}')
 
     elif event['_event'] == 'NNet.Game.SControlGroupUpdateEvent':
         current_event = ControlGroupEvent(game, event)
+        logger.debug(f'Created new ControlGroupEvent at {event["_gameloop"]}')
 
     elif event['_event'] == 'NNet.Replay.Tracker.SUpgradeEvent':
         current_event = UpgradeEvent(game, event)
+        logger.debug(f'Created new UpgradeEvent at {event["_gameloop"]}')
 
     elif event['_event'] == 'NNet.Game.SCameraUpdateEvent':
-        current_event = CameraUpdateEvent(game, event)
+        current_event = CameraEvent(game, event)
+        logger.debug(f'Created new CameraEvent at {event["_gameloop"]}')
 
     elif event['_event'] == 'NNet.Replay.Tracker.SPlayerStatsEvent':
         current_event = PlayerStatsEvent(summary_stats, game, event)
+        logger.debug(f'Created new PlayerStatsEvent at {event["_gameloop"]}')
 
     else:
         current_event = None
+        logger.warning(f'Event of type {event["_event"]} is not supported')
 
     return current_event
